/* NBD client library in userspace
 * WARNING: THIS FILE IS GENERATED FROM
 * generator/generator
 * ANY CHANGES YOU MAKE TO THIS FILE WILL BE LOST.
 *
 * Copyright (C) 2013-2019 Red Hat Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include <config.h>

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>

#include <pthread.h>

#include "libnbd.h"
#include "internal.h"

int
nbd_set_debug (struct nbd_handle *h, bool debug)
{
  int ret;

  nbd_internal_reset_error ("nbd_set_debug");

  pthread_mutex_lock (&h->lock);
  ret = nbd_unlocked_set_debug (h, debug);
  pthread_mutex_unlock (&h->lock);
  return ret;
}

int
nbd_get_debug (struct nbd_handle *h)
{
  int ret;

  nbd_internal_reset_error ("nbd_get_debug");

  pthread_mutex_lock (&h->lock);
  ret = nbd_unlocked_get_debug (h);
  pthread_mutex_unlock (&h->lock);
  return ret;
}

int
nbd_set_debug_callback (struct nbd_handle *h, void *data, int (*debug_fn) (void *data, const char *context, const char *msg))
{
  int ret;

  nbd_internal_reset_error ("nbd_set_debug_callback");

  pthread_mutex_lock (&h->lock);
  ret = nbd_unlocked_set_debug_callback (h, data, debug_fn);
  pthread_mutex_unlock (&h->lock);
  return ret;
}

int
nbd_set_export_name (struct nbd_handle *h, const char *export_name)
{
  int ret;

  nbd_internal_reset_error ("nbd_set_export_name");

  /* We can check the state outside the handle lock because the
   * the state is atomic.
   */
  enum state state = get_state (h);
  if (!(nbd_internal_is_state_created (state))) {
    set_error (nbd_internal_is_state_created (state) ? ENOTCONN : EINVAL,
               "invalid state: %s: the handle must be %s",
               nbd_internal_state_short_string (state),
               "newly created");
    return -1;
  }

  pthread_mutex_lock (&h->lock);
  ret = nbd_unlocked_set_export_name (h, export_name);
  pthread_mutex_unlock (&h->lock);
  return ret;
}

char *
nbd_get_export_name (struct nbd_handle *h)
{
  char *ret;

  nbd_internal_reset_error ("nbd_get_export_name");

  pthread_mutex_lock (&h->lock);
  ret = nbd_unlocked_get_export_name (h);
  pthread_mutex_unlock (&h->lock);
  return ret;
}

int
nbd_set_tls (struct nbd_handle *h, int tls)
{
  int ret;

  nbd_internal_reset_error ("nbd_set_tls");

  /* We can check the state outside the handle lock because the
   * the state is atomic.
   */
  enum state state = get_state (h);
  if (!(nbd_internal_is_state_created (state))) {
    set_error (nbd_internal_is_state_created (state) ? ENOTCONN : EINVAL,
               "invalid state: %s: the handle must be %s",
               nbd_internal_state_short_string (state),
               "newly created");
    return -1;
  }

  pthread_mutex_lock (&h->lock);
  ret = nbd_unlocked_set_tls (h, tls);
  pthread_mutex_unlock (&h->lock);
  return ret;
}

int
nbd_get_tls (struct nbd_handle *h)
{
  int ret;

  nbd_internal_reset_error ("nbd_get_tls");

  pthread_mutex_lock (&h->lock);
  ret = nbd_unlocked_get_tls (h);
  pthread_mutex_unlock (&h->lock);
  return ret;
}

int
nbd_set_tls_certificates (struct nbd_handle *h, const char *dir)
{
  int ret;

  nbd_internal_reset_error ("nbd_set_tls_certificates");

  /* We can check the state outside the handle lock because the
   * the state is atomic.
   */
  enum state state = get_state (h);
  if (!(nbd_internal_is_state_created (state))) {
    set_error (nbd_internal_is_state_created (state) ? ENOTCONN : EINVAL,
               "invalid state: %s: the handle must be %s",
               nbd_internal_state_short_string (state),
               "newly created");
    return -1;
  }

  pthread_mutex_lock (&h->lock);
  ret = nbd_unlocked_set_tls_certificates (h, dir);
  pthread_mutex_unlock (&h->lock);
  return ret;
}

int
nbd_set_tls_verify_peer (struct nbd_handle *h, bool verify)
{
  int ret;

  nbd_internal_reset_error ("nbd_set_tls_verify_peer");

  /* We can check the state outside the handle lock because the
   * the state is atomic.
   */
  enum state state = get_state (h);
  if (!(nbd_internal_is_state_created (state))) {
    set_error (nbd_internal_is_state_created (state) ? ENOTCONN : EINVAL,
               "invalid state: %s: the handle must be %s",
               nbd_internal_state_short_string (state),
               "newly created");
    return -1;
  }

  pthread_mutex_lock (&h->lock);
  ret = nbd_unlocked_set_tls_verify_peer (h, verify);
  pthread_mutex_unlock (&h->lock);
  return ret;
}

int
nbd_get_tls_verify_peer (struct nbd_handle *h)
{
  int ret;

  nbd_internal_reset_error ("nbd_get_tls_verify_peer");

  pthread_mutex_lock (&h->lock);
  ret = nbd_unlocked_get_tls_verify_peer (h);
  pthread_mutex_unlock (&h->lock);
  return ret;
}

int
nbd_set_tls_username (struct nbd_handle *h, const char *username)
{
  int ret;

  nbd_internal_reset_error ("nbd_set_tls_username");

  /* We can check the state outside the handle lock because the
   * the state is atomic.
   */
  enum state state = get_state (h);
  if (!(nbd_internal_is_state_created (state))) {
    set_error (nbd_internal_is_state_created (state) ? ENOTCONN : EINVAL,
               "invalid state: %s: the handle must be %s",
               nbd_internal_state_short_string (state),
               "newly created");
    return -1;
  }

  pthread_mutex_lock (&h->lock);
  ret = nbd_unlocked_set_tls_username (h, username);
  pthread_mutex_unlock (&h->lock);
  return ret;
}

char *
nbd_get_tls_username (struct nbd_handle *h)
{
  char *ret;

  nbd_internal_reset_error ("nbd_get_tls_username");

  pthread_mutex_lock (&h->lock);
  ret = nbd_unlocked_get_tls_username (h);
  pthread_mutex_unlock (&h->lock);
  return ret;
}

int
nbd_set_tls_psk_file (struct nbd_handle *h, const char *filename)
{
  int ret;

  nbd_internal_reset_error ("nbd_set_tls_psk_file");

  /* We can check the state outside the handle lock because the
   * the state is atomic.
   */
  enum state state = get_state (h);
  if (!(nbd_internal_is_state_created (state))) {
    set_error (nbd_internal_is_state_created (state) ? ENOTCONN : EINVAL,
               "invalid state: %s: the handle must be %s",
               nbd_internal_state_short_string (state),
               "newly created");
    return -1;
  }

  pthread_mutex_lock (&h->lock);
  ret = nbd_unlocked_set_tls_psk_file (h, filename);
  pthread_mutex_unlock (&h->lock);
  return ret;
}

int
nbd_add_meta_context (struct nbd_handle *h, const char *name)
{
  int ret;

  nbd_internal_reset_error ("nbd_add_meta_context");

  /* We can check the state outside the handle lock because the
   * the state is atomic.
   */
  enum state state = get_state (h);
  if (!(nbd_internal_is_state_created (state))) {
    set_error (nbd_internal_is_state_created (state) ? ENOTCONN : EINVAL,
               "invalid state: %s: the handle must be %s",
               nbd_internal_state_short_string (state),
               "newly created");
    return -1;
  }

  pthread_mutex_lock (&h->lock);
  ret = nbd_unlocked_add_meta_context (h, name);
  pthread_mutex_unlock (&h->lock);
  return ret;
}

int
nbd_connect_uri (struct nbd_handle *h, const char *uri)
{
  int ret;

  nbd_internal_reset_error ("nbd_connect_uri");

  /* We can check the state outside the handle lock because the
   * the state is atomic.
   */
  enum state state = get_state (h);
  if (!(nbd_internal_is_state_created (state))) {
    set_error (nbd_internal_is_state_created (state) ? ENOTCONN : EINVAL,
               "invalid state: %s: the handle must be %s",
               nbd_internal_state_short_string (state),
               "newly created");
    return -1;
  }

  pthread_mutex_lock (&h->lock);
  ret = nbd_unlocked_connect_uri (h, uri);
  pthread_mutex_unlock (&h->lock);
  return ret;
}

int
nbd_connect_unix (struct nbd_handle *h, const char *unixsocket)
{
  int ret;

  nbd_internal_reset_error ("nbd_connect_unix");

  /* We can check the state outside the handle lock because the
   * the state is atomic.
   */
  enum state state = get_state (h);
  if (!(nbd_internal_is_state_created (state))) {
    set_error (nbd_internal_is_state_created (state) ? ENOTCONN : EINVAL,
               "invalid state: %s: the handle must be %s",
               nbd_internal_state_short_string (state),
               "newly created");
    return -1;
  }

  pthread_mutex_lock (&h->lock);
  ret = nbd_unlocked_connect_unix (h, unixsocket);
  pthread_mutex_unlock (&h->lock);
  return ret;
}

int
nbd_connect_tcp (struct nbd_handle *h, const char *hostname, const char *port)
{
  int ret;

  nbd_internal_reset_error ("nbd_connect_tcp");

  /* We can check the state outside the handle lock because the
   * the state is atomic.
   */
  enum state state = get_state (h);
  if (!(nbd_internal_is_state_created (state))) {
    set_error (nbd_internal_is_state_created (state) ? ENOTCONN : EINVAL,
               "invalid state: %s: the handle must be %s",
               nbd_internal_state_short_string (state),
               "newly created");
    return -1;
  }

  pthread_mutex_lock (&h->lock);
  ret = nbd_unlocked_connect_tcp (h, hostname, port);
  pthread_mutex_unlock (&h->lock);
  return ret;
}

int
nbd_connect_command (struct nbd_handle *h, char **argv)
{
  int ret;

  nbd_internal_reset_error ("nbd_connect_command");

  /* We can check the state outside the handle lock because the
   * the state is atomic.
   */
  enum state state = get_state (h);
  if (!(nbd_internal_is_state_created (state))) {
    set_error (nbd_internal_is_state_created (state) ? ENOTCONN : EINVAL,
               "invalid state: %s: the handle must be %s",
               nbd_internal_state_short_string (state),
               "newly created");
    return -1;
  }

  pthread_mutex_lock (&h->lock);
  ret = nbd_unlocked_connect_command (h, argv);
  pthread_mutex_unlock (&h->lock);
  return ret;
}

int
nbd_read_only (struct nbd_handle *h)
{
  int ret;

  nbd_internal_reset_error ("nbd_read_only");

  /* We can check the state outside the handle lock because the
   * the state is atomic.
   */
  enum state state = get_state (h);
  if (!(nbd_internal_is_state_ready (state) || nbd_internal_is_state_processing (state) ||
        nbd_internal_is_state_closed (state))) {
    set_error (nbd_internal_is_state_created (state) ? ENOTCONN : EINVAL,
               "invalid state: %s: the handle must be %s",
               nbd_internal_state_short_string (state),
               "connected and finished handshaking with the server, or closed");
    return -1;
  }

  pthread_mutex_lock (&h->lock);
  ret = nbd_unlocked_read_only (h);
  pthread_mutex_unlock (&h->lock);
  return ret;
}

int
nbd_can_flush (struct nbd_handle *h)
{
  int ret;

  nbd_internal_reset_error ("nbd_can_flush");

  /* We can check the state outside the handle lock because the
   * the state is atomic.
   */
  enum state state = get_state (h);
  if (!(nbd_internal_is_state_ready (state) || nbd_internal_is_state_processing (state) ||
        nbd_internal_is_state_closed (state))) {
    set_error (nbd_internal_is_state_created (state) ? ENOTCONN : EINVAL,
               "invalid state: %s: the handle must be %s",
               nbd_internal_state_short_string (state),
               "connected and finished handshaking with the server, or closed");
    return -1;
  }

  pthread_mutex_lock (&h->lock);
  ret = nbd_unlocked_can_flush (h);
  pthread_mutex_unlock (&h->lock);
  return ret;
}

int
nbd_can_fua (struct nbd_handle *h)
{
  int ret;

  nbd_internal_reset_error ("nbd_can_fua");

  /* We can check the state outside the handle lock because the
   * the state is atomic.
   */
  enum state state = get_state (h);
  if (!(nbd_internal_is_state_ready (state) || nbd_internal_is_state_processing (state) ||
        nbd_internal_is_state_closed (state))) {
    set_error (nbd_internal_is_state_created (state) ? ENOTCONN : EINVAL,
               "invalid state: %s: the handle must be %s",
               nbd_internal_state_short_string (state),
               "connected and finished handshaking with the server, or closed");
    return -1;
  }

  pthread_mutex_lock (&h->lock);
  ret = nbd_unlocked_can_fua (h);
  pthread_mutex_unlock (&h->lock);
  return ret;
}

int
nbd_is_rotational (struct nbd_handle *h)
{
  int ret;

  nbd_internal_reset_error ("nbd_is_rotational");

  /* We can check the state outside the handle lock because the
   * the state is atomic.
   */
  enum state state = get_state (h);
  if (!(nbd_internal_is_state_ready (state) || nbd_internal_is_state_processing (state) ||
        nbd_internal_is_state_closed (state))) {
    set_error (nbd_internal_is_state_created (state) ? ENOTCONN : EINVAL,
               "invalid state: %s: the handle must be %s",
               nbd_internal_state_short_string (state),
               "connected and finished handshaking with the server, or closed");
    return -1;
  }

  pthread_mutex_lock (&h->lock);
  ret = nbd_unlocked_is_rotational (h);
  pthread_mutex_unlock (&h->lock);
  return ret;
}

int
nbd_can_trim (struct nbd_handle *h)
{
  int ret;

  nbd_internal_reset_error ("nbd_can_trim");

  /* We can check the state outside the handle lock because the
   * the state is atomic.
   */
  enum state state = get_state (h);
  if (!(nbd_internal_is_state_ready (state) || nbd_internal_is_state_processing (state) ||
        nbd_internal_is_state_closed (state))) {
    set_error (nbd_internal_is_state_created (state) ? ENOTCONN : EINVAL,
               "invalid state: %s: the handle must be %s",
               nbd_internal_state_short_string (state),
               "connected and finished handshaking with the server, or closed");
    return -1;
  }

  pthread_mutex_lock (&h->lock);
  ret = nbd_unlocked_can_trim (h);
  pthread_mutex_unlock (&h->lock);
  return ret;
}

int
nbd_can_zero (struct nbd_handle *h)
{
  int ret;

  nbd_internal_reset_error ("nbd_can_zero");

  /* We can check the state outside the handle lock because the
   * the state is atomic.
   */
  enum state state = get_state (h);
  if (!(nbd_internal_is_state_ready (state) || nbd_internal_is_state_processing (state) ||
        nbd_internal_is_state_closed (state))) {
    set_error (nbd_internal_is_state_created (state) ? ENOTCONN : EINVAL,
               "invalid state: %s: the handle must be %s",
               nbd_internal_state_short_string (state),
               "connected and finished handshaking with the server, or closed");
    return -1;
  }

  pthread_mutex_lock (&h->lock);
  ret = nbd_unlocked_can_zero (h);
  pthread_mutex_unlock (&h->lock);
  return ret;
}

int
nbd_can_multi_conn (struct nbd_handle *h)
{
  int ret;

  nbd_internal_reset_error ("nbd_can_multi_conn");

  /* We can check the state outside the handle lock because the
   * the state is atomic.
   */
  enum state state = get_state (h);
  if (!(nbd_internal_is_state_ready (state) || nbd_internal_is_state_processing (state) ||
        nbd_internal_is_state_closed (state))) {
    set_error (nbd_internal_is_state_created (state) ? ENOTCONN : EINVAL,
               "invalid state: %s: the handle must be %s",
               nbd_internal_state_short_string (state),
               "connected and finished handshaking with the server, or closed");
    return -1;
  }

  pthread_mutex_lock (&h->lock);
  ret = nbd_unlocked_can_multi_conn (h);
  pthread_mutex_unlock (&h->lock);
  return ret;
}

int
nbd_can_cache (struct nbd_handle *h)
{
  int ret;

  nbd_internal_reset_error ("nbd_can_cache");

  /* We can check the state outside the handle lock because the
   * the state is atomic.
   */
  enum state state = get_state (h);
  if (!(nbd_internal_is_state_ready (state) || nbd_internal_is_state_processing (state) ||
        nbd_internal_is_state_closed (state))) {
    set_error (nbd_internal_is_state_created (state) ? ENOTCONN : EINVAL,
               "invalid state: %s: the handle must be %s",
               nbd_internal_state_short_string (state),
               "connected and finished handshaking with the server, or closed");
    return -1;
  }

  pthread_mutex_lock (&h->lock);
  ret = nbd_unlocked_can_cache (h);
  pthread_mutex_unlock (&h->lock);
  return ret;
}

int
nbd_can_meta_context (struct nbd_handle *h, const char *metacontext)
{
  int ret;

  nbd_internal_reset_error ("nbd_can_meta_context");

  /* We can check the state outside the handle lock because the
   * the state is atomic.
   */
  enum state state = get_state (h);
  if (!(nbd_internal_is_state_ready (state) || nbd_internal_is_state_processing (state) ||
        nbd_internal_is_state_closed (state))) {
    set_error (nbd_internal_is_state_created (state) ? ENOTCONN : EINVAL,
               "invalid state: %s: the handle must be %s",
               nbd_internal_state_short_string (state),
               "connected and finished handshaking with the server, or closed");
    return -1;
  }

  pthread_mutex_lock (&h->lock);
  ret = nbd_unlocked_can_meta_context (h, metacontext);
  pthread_mutex_unlock (&h->lock);
  return ret;
}

int64_t
nbd_get_size (struct nbd_handle *h)
{
  int64_t ret;

  nbd_internal_reset_error ("nbd_get_size");

  /* We can check the state outside the handle lock because the
   * the state is atomic.
   */
  enum state state = get_state (h);
  if (!(nbd_internal_is_state_ready (state) || nbd_internal_is_state_processing (state) ||
        nbd_internal_is_state_closed (state))) {
    set_error (nbd_internal_is_state_created (state) ? ENOTCONN : EINVAL,
               "invalid state: %s: the handle must be %s",
               nbd_internal_state_short_string (state),
               "connected and finished handshaking with the server, or closed");
    return -1;
  }

  pthread_mutex_lock (&h->lock);
  ret = nbd_unlocked_get_size (h);
  pthread_mutex_unlock (&h->lock);
  return ret;
}

int
nbd_pread (struct nbd_handle *h, void *buf, size_t count, uint64_t offset, uint32_t flags)
{
  int ret;

  nbd_internal_reset_error ("nbd_pread");

  /* We can check the state outside the handle lock because the
   * the state is atomic.
   */
  enum state state = get_state (h);
  if (!(nbd_internal_is_state_ready (state) || nbd_internal_is_state_processing (state))) {
    set_error (nbd_internal_is_state_created (state) ? ENOTCONN : EINVAL,
               "invalid state: %s: the handle must be %s",
               nbd_internal_state_short_string (state),
               "connected and finished handshaking with the server");
    return -1;
  }

  pthread_mutex_lock (&h->lock);
  ret = nbd_unlocked_pread (h, buf, count, offset, flags);
  pthread_mutex_unlock (&h->lock);
  return ret;
}

int
nbd_pwrite (struct nbd_handle *h, const void *buf, size_t count, uint64_t offset, uint32_t flags)
{
  int ret;

  nbd_internal_reset_error ("nbd_pwrite");

  /* We can check the state outside the handle lock because the
   * the state is atomic.
   */
  enum state state = get_state (h);
  if (!(nbd_internal_is_state_ready (state) || nbd_internal_is_state_processing (state))) {
    set_error (nbd_internal_is_state_created (state) ? ENOTCONN : EINVAL,
               "invalid state: %s: the handle must be %s",
               nbd_internal_state_short_string (state),
               "connected and finished handshaking with the server");
    return -1;
  }

  pthread_mutex_lock (&h->lock);
  ret = nbd_unlocked_pwrite (h, buf, count, offset, flags);
  pthread_mutex_unlock (&h->lock);
  return ret;
}

int
nbd_shutdown (struct nbd_handle *h)
{
  int ret;

  nbd_internal_reset_error ("nbd_shutdown");

  /* We can check the state outside the handle lock because the
   * the state is atomic.
   */
  enum state state = get_state (h);
  if (!(nbd_internal_is_state_ready (state) || nbd_internal_is_state_processing (state))) {
    set_error (nbd_internal_is_state_created (state) ? ENOTCONN : EINVAL,
               "invalid state: %s: the handle must be %s",
               nbd_internal_state_short_string (state),
               "connected and finished handshaking with the server");
    return -1;
  }

  pthread_mutex_lock (&h->lock);
  ret = nbd_unlocked_shutdown (h);
  pthread_mutex_unlock (&h->lock);
  return ret;
}

int
nbd_flush (struct nbd_handle *h, uint32_t flags)
{
  int ret;

  nbd_internal_reset_error ("nbd_flush");

  /* We can check the state outside the handle lock because the
   * the state is atomic.
   */
  enum state state = get_state (h);
  if (!(nbd_internal_is_state_ready (state) || nbd_internal_is_state_processing (state))) {
    set_error (nbd_internal_is_state_created (state) ? ENOTCONN : EINVAL,
               "invalid state: %s: the handle must be %s",
               nbd_internal_state_short_string (state),
               "connected and finished handshaking with the server");
    return -1;
  }

  pthread_mutex_lock (&h->lock);
  ret = nbd_unlocked_flush (h, flags);
  pthread_mutex_unlock (&h->lock);
  return ret;
}

int
nbd_trim (struct nbd_handle *h, uint64_t count, uint64_t offset, uint32_t flags)
{
  int ret;

  nbd_internal_reset_error ("nbd_trim");

  /* We can check the state outside the handle lock because the
   * the state is atomic.
   */
  enum state state = get_state (h);
  if (!(nbd_internal_is_state_ready (state) || nbd_internal_is_state_processing (state))) {
    set_error (nbd_internal_is_state_created (state) ? ENOTCONN : EINVAL,
               "invalid state: %s: the handle must be %s",
               nbd_internal_state_short_string (state),
               "connected and finished handshaking with the server");
    return -1;
  }

  pthread_mutex_lock (&h->lock);
  ret = nbd_unlocked_trim (h, count, offset, flags);
  pthread_mutex_unlock (&h->lock);
  return ret;
}

int
nbd_cache (struct nbd_handle *h, uint64_t count, uint64_t offset, uint32_t flags)
{
  int ret;

  nbd_internal_reset_error ("nbd_cache");

  /* We can check the state outside the handle lock because the
   * the state is atomic.
   */
  enum state state = get_state (h);
  if (!(nbd_internal_is_state_ready (state) || nbd_internal_is_state_processing (state))) {
    set_error (nbd_internal_is_state_created (state) ? ENOTCONN : EINVAL,
               "invalid state: %s: the handle must be %s",
               nbd_internal_state_short_string (state),
               "connected and finished handshaking with the server");
    return -1;
  }

  pthread_mutex_lock (&h->lock);
  ret = nbd_unlocked_cache (h, count, offset, flags);
  pthread_mutex_unlock (&h->lock);
  return ret;
}

int
nbd_zero (struct nbd_handle *h, uint64_t count, uint64_t offset, uint32_t flags)
{
  int ret;

  nbd_internal_reset_error ("nbd_zero");

  /* We can check the state outside the handle lock because the
   * the state is atomic.
   */
  enum state state = get_state (h);
  if (!(nbd_internal_is_state_ready (state) || nbd_internal_is_state_processing (state))) {
    set_error (nbd_internal_is_state_created (state) ? ENOTCONN : EINVAL,
               "invalid state: %s: the handle must be %s",
               nbd_internal_state_short_string (state),
               "connected and finished handshaking with the server");
    return -1;
  }

  pthread_mutex_lock (&h->lock);
  ret = nbd_unlocked_zero (h, count, offset, flags);
  pthread_mutex_unlock (&h->lock);
  return ret;
}

int
nbd_block_status (struct nbd_handle *h, uint64_t count, uint64_t offset, void *data, int (*extent) (void *data, const char *metacontext, uint64_t offset, uint32_t *entries, size_t nr_entries), uint32_t flags)
{
  int ret;

  nbd_internal_reset_error ("nbd_block_status");

  /* We can check the state outside the handle lock because the
   * the state is atomic.
   */
  enum state state = get_state (h);
  if (!(nbd_internal_is_state_ready (state) || nbd_internal_is_state_processing (state))) {
    set_error (nbd_internal_is_state_created (state) ? ENOTCONN : EINVAL,
               "invalid state: %s: the handle must be %s",
               nbd_internal_state_short_string (state),
               "connected and finished handshaking with the server");
    return -1;
  }

  pthread_mutex_lock (&h->lock);
  ret = nbd_unlocked_block_status (h, count, offset, data, extent, flags);
  pthread_mutex_unlock (&h->lock);
  return ret;
}

int
nbd_poll (struct nbd_handle *h, int timeout)
{
  int ret;

  nbd_internal_reset_error ("nbd_poll");

  pthread_mutex_lock (&h->lock);
  ret = nbd_unlocked_poll (h, timeout);
  pthread_mutex_unlock (&h->lock);
  return ret;
}

int
nbd_aio_connect (struct nbd_handle *h, const struct sockaddr *addr, socklen_t addrlen)
{
  int ret;

  nbd_internal_reset_error ("nbd_aio_connect");

  /* We can check the state outside the handle lock because the
   * the state is atomic.
   */
  enum state state = get_state (h);
  if (!(nbd_internal_is_state_created (state))) {
    set_error (nbd_internal_is_state_created (state) ? ENOTCONN : EINVAL,
               "invalid state: %s: the handle must be %s",
               nbd_internal_state_short_string (state),
               "newly created");
    return -1;
  }

  pthread_mutex_lock (&h->lock);
  ret = nbd_unlocked_aio_connect (h, addr, addrlen);
  pthread_mutex_unlock (&h->lock);
  return ret;
}

int
nbd_aio_connect_uri (struct nbd_handle *h, const char *uri)
{
  int ret;

  nbd_internal_reset_error ("nbd_aio_connect_uri");

  /* We can check the state outside the handle lock because the
   * the state is atomic.
   */
  enum state state = get_state (h);
  if (!(nbd_internal_is_state_created (state))) {
    set_error (nbd_internal_is_state_created (state) ? ENOTCONN : EINVAL,
               "invalid state: %s: the handle must be %s",
               nbd_internal_state_short_string (state),
               "newly created");
    return -1;
  }

  pthread_mutex_lock (&h->lock);
  ret = nbd_unlocked_aio_connect_uri (h, uri);
  pthread_mutex_unlock (&h->lock);
  return ret;
}

int
nbd_aio_connect_unix (struct nbd_handle *h, const char *unixsocket)
{
  int ret;

  nbd_internal_reset_error ("nbd_aio_connect_unix");

  /* We can check the state outside the handle lock because the
   * the state is atomic.
   */
  enum state state = get_state (h);
  if (!(nbd_internal_is_state_created (state))) {
    set_error (nbd_internal_is_state_created (state) ? ENOTCONN : EINVAL,
               "invalid state: %s: the handle must be %s",
               nbd_internal_state_short_string (state),
               "newly created");
    return -1;
  }

  pthread_mutex_lock (&h->lock);
  ret = nbd_unlocked_aio_connect_unix (h, unixsocket);
  pthread_mutex_unlock (&h->lock);
  return ret;
}

int
nbd_aio_connect_tcp (struct nbd_handle *h, const char *hostname, const char *port)
{
  int ret;

  nbd_internal_reset_error ("nbd_aio_connect_tcp");

  /* We can check the state outside the handle lock because the
   * the state is atomic.
   */
  enum state state = get_state (h);
  if (!(nbd_internal_is_state_created (state))) {
    set_error (nbd_internal_is_state_created (state) ? ENOTCONN : EINVAL,
               "invalid state: %s: the handle must be %s",
               nbd_internal_state_short_string (state),
               "newly created");
    return -1;
  }

  pthread_mutex_lock (&h->lock);
  ret = nbd_unlocked_aio_connect_tcp (h, hostname, port);
  pthread_mutex_unlock (&h->lock);
  return ret;
}

int
nbd_aio_connect_command (struct nbd_handle *h, char **argv)
{
  int ret;

  nbd_internal_reset_error ("nbd_aio_connect_command");

  /* We can check the state outside the handle lock because the
   * the state is atomic.
   */
  enum state state = get_state (h);
  if (!(nbd_internal_is_state_created (state))) {
    set_error (nbd_internal_is_state_created (state) ? ENOTCONN : EINVAL,
               "invalid state: %s: the handle must be %s",
               nbd_internal_state_short_string (state),
               "newly created");
    return -1;
  }

  pthread_mutex_lock (&h->lock);
  ret = nbd_unlocked_aio_connect_command (h, argv);
  pthread_mutex_unlock (&h->lock);
  return ret;
}

int64_t
nbd_aio_pread (struct nbd_handle *h, void *buf, size_t count, uint64_t offset, uint32_t flags)
{
  int64_t ret;

  nbd_internal_reset_error ("nbd_aio_pread");

  /* We can check the state outside the handle lock because the
   * the state is atomic.
   */
  enum state state = get_state (h);
  if (!(nbd_internal_is_state_ready (state) || nbd_internal_is_state_processing (state))) {
    set_error (nbd_internal_is_state_created (state) ? ENOTCONN : EINVAL,
               "invalid state: %s: the handle must be %s",
               nbd_internal_state_short_string (state),
               "connected and finished handshaking with the server");
    return -1;
  }

  pthread_mutex_lock (&h->lock);
  ret = nbd_unlocked_aio_pread (h, buf, count, offset, flags);
  pthread_mutex_unlock (&h->lock);
  return ret;
}

int64_t
nbd_aio_pwrite (struct nbd_handle *h, const void *buf, size_t count, uint64_t offset, uint32_t flags)
{
  int64_t ret;

  nbd_internal_reset_error ("nbd_aio_pwrite");

  /* We can check the state outside the handle lock because the
   * the state is atomic.
   */
  enum state state = get_state (h);
  if (!(nbd_internal_is_state_ready (state) || nbd_internal_is_state_processing (state))) {
    set_error (nbd_internal_is_state_created (state) ? ENOTCONN : EINVAL,
               "invalid state: %s: the handle must be %s",
               nbd_internal_state_short_string (state),
               "connected and finished handshaking with the server");
    return -1;
  }

  pthread_mutex_lock (&h->lock);
  ret = nbd_unlocked_aio_pwrite (h, buf, count, offset, flags);
  pthread_mutex_unlock (&h->lock);
  return ret;
}

int
nbd_aio_disconnect (struct nbd_handle *h, uint32_t flags)
{
  int ret;

  nbd_internal_reset_error ("nbd_aio_disconnect");

  /* We can check the state outside the handle lock because the
   * the state is atomic.
   */
  enum state state = get_state (h);
  if (!(nbd_internal_is_state_ready (state) || nbd_internal_is_state_processing (state))) {
    set_error (nbd_internal_is_state_created (state) ? ENOTCONN : EINVAL,
               "invalid state: %s: the handle must be %s",
               nbd_internal_state_short_string (state),
               "connected and finished handshaking with the server");
    return -1;
  }

  pthread_mutex_lock (&h->lock);
  ret = nbd_unlocked_aio_disconnect (h, flags);
  pthread_mutex_unlock (&h->lock);
  return ret;
}

int64_t
nbd_aio_flush (struct nbd_handle *h, uint32_t flags)
{
  int64_t ret;

  nbd_internal_reset_error ("nbd_aio_flush");

  /* We can check the state outside the handle lock because the
   * the state is atomic.
   */
  enum state state = get_state (h);
  if (!(nbd_internal_is_state_ready (state) || nbd_internal_is_state_processing (state))) {
    set_error (nbd_internal_is_state_created (state) ? ENOTCONN : EINVAL,
               "invalid state: %s: the handle must be %s",
               nbd_internal_state_short_string (state),
               "connected and finished handshaking with the server");
    return -1;
  }

  pthread_mutex_lock (&h->lock);
  ret = nbd_unlocked_aio_flush (h, flags);
  pthread_mutex_unlock (&h->lock);
  return ret;
}

int64_t
nbd_aio_trim (struct nbd_handle *h, uint64_t count, uint64_t offset, uint32_t flags)
{
  int64_t ret;

  nbd_internal_reset_error ("nbd_aio_trim");

  /* We can check the state outside the handle lock because the
   * the state is atomic.
   */
  enum state state = get_state (h);
  if (!(nbd_internal_is_state_ready (state) || nbd_internal_is_state_processing (state))) {
    set_error (nbd_internal_is_state_created (state) ? ENOTCONN : EINVAL,
               "invalid state: %s: the handle must be %s",
               nbd_internal_state_short_string (state),
               "connected and finished handshaking with the server");
    return -1;
  }

  pthread_mutex_lock (&h->lock);
  ret = nbd_unlocked_aio_trim (h, count, offset, flags);
  pthread_mutex_unlock (&h->lock);
  return ret;
}

int64_t
nbd_aio_cache (struct nbd_handle *h, uint64_t count, uint64_t offset, uint32_t flags)
{
  int64_t ret;

  nbd_internal_reset_error ("nbd_aio_cache");

  /* We can check the state outside the handle lock because the
   * the state is atomic.
   */
  enum state state = get_state (h);
  if (!(nbd_internal_is_state_ready (state) || nbd_internal_is_state_processing (state))) {
    set_error (nbd_internal_is_state_created (state) ? ENOTCONN : EINVAL,
               "invalid state: %s: the handle must be %s",
               nbd_internal_state_short_string (state),
               "connected and finished handshaking with the server");
    return -1;
  }

  pthread_mutex_lock (&h->lock);
  ret = nbd_unlocked_aio_cache (h, count, offset, flags);
  pthread_mutex_unlock (&h->lock);
  return ret;
}

int64_t
nbd_aio_zero (struct nbd_handle *h, uint64_t count, uint64_t offset, uint32_t flags)
{
  int64_t ret;

  nbd_internal_reset_error ("nbd_aio_zero");

  /* We can check the state outside the handle lock because the
   * the state is atomic.
   */
  enum state state = get_state (h);
  if (!(nbd_internal_is_state_ready (state) || nbd_internal_is_state_processing (state))) {
    set_error (nbd_internal_is_state_created (state) ? ENOTCONN : EINVAL,
               "invalid state: %s: the handle must be %s",
               nbd_internal_state_short_string (state),
               "connected and finished handshaking with the server");
    return -1;
  }

  pthread_mutex_lock (&h->lock);
  ret = nbd_unlocked_aio_zero (h, count, offset, flags);
  pthread_mutex_unlock (&h->lock);
  return ret;
}

int64_t
nbd_aio_block_status (struct nbd_handle *h, uint64_t count, uint64_t offset, void *data, int (*extent) (void *data, const char *metacontext, uint64_t offset, uint32_t *entries, size_t nr_entries), uint32_t flags)
{
  int64_t ret;

  nbd_internal_reset_error ("nbd_aio_block_status");

  /* We can check the state outside the handle lock because the
   * the state is atomic.
   */
  enum state state = get_state (h);
  if (!(nbd_internal_is_state_ready (state) || nbd_internal_is_state_processing (state))) {
    set_error (nbd_internal_is_state_created (state) ? ENOTCONN : EINVAL,
               "invalid state: %s: the handle must be %s",
               nbd_internal_state_short_string (state),
               "connected and finished handshaking with the server");
    return -1;
  }

  pthread_mutex_lock (&h->lock);
  ret = nbd_unlocked_aio_block_status (h, count, offset, data, extent, flags);
  pthread_mutex_unlock (&h->lock);
  return ret;
}

int
nbd_aio_get_fd (struct nbd_handle *h)
{
  int ret;

  nbd_internal_reset_error ("nbd_aio_get_fd");

  pthread_mutex_lock (&h->lock);
  ret = nbd_unlocked_aio_get_fd (h);
  pthread_mutex_unlock (&h->lock);
  return ret;
}

int
nbd_aio_get_direction (struct nbd_handle *h)
{
  int ret;

  ret = nbd_unlocked_aio_get_direction (h);
  return ret;
}

int
nbd_aio_notify_read (struct nbd_handle *h)
{
  int ret;

  nbd_internal_reset_error ("nbd_aio_notify_read");

  pthread_mutex_lock (&h->lock);
  ret = nbd_unlocked_aio_notify_read (h);
  pthread_mutex_unlock (&h->lock);
  return ret;
}

int
nbd_aio_notify_write (struct nbd_handle *h)
{
  int ret;

  nbd_internal_reset_error ("nbd_aio_notify_write");

  pthread_mutex_lock (&h->lock);
  ret = nbd_unlocked_aio_notify_write (h);
  pthread_mutex_unlock (&h->lock);
  return ret;
}

int
nbd_aio_is_created (struct nbd_handle *h)
{
  int ret;

  ret = nbd_unlocked_aio_is_created (h);
  return ret;
}

int
nbd_aio_is_connecting (struct nbd_handle *h)
{
  int ret;

  ret = nbd_unlocked_aio_is_connecting (h);
  return ret;
}

int
nbd_aio_is_ready (struct nbd_handle *h)
{
  int ret;

  ret = nbd_unlocked_aio_is_ready (h);
  return ret;
}

int
nbd_aio_is_processing (struct nbd_handle *h)
{
  int ret;

  ret = nbd_unlocked_aio_is_processing (h);
  return ret;
}

int
nbd_aio_is_dead (struct nbd_handle *h)
{
  int ret;

  ret = nbd_unlocked_aio_is_dead (h);
  return ret;
}

int
nbd_aio_is_closed (struct nbd_handle *h)
{
  int ret;

  ret = nbd_unlocked_aio_is_closed (h);
  return ret;
}

int
nbd_aio_command_completed (struct nbd_handle *h, int64_t handle)
{
  int ret;

  nbd_internal_reset_error ("nbd_aio_command_completed");

  pthread_mutex_lock (&h->lock);
  ret = nbd_unlocked_aio_command_completed (h, handle);
  pthread_mutex_unlock (&h->lock);
  return ret;
}

int64_t
nbd_aio_peek_command_completed (struct nbd_handle *h)
{
  int64_t ret;

  nbd_internal_reset_error ("nbd_aio_peek_command_completed");

  pthread_mutex_lock (&h->lock);
  ret = nbd_unlocked_aio_peek_command_completed (h);
  pthread_mutex_unlock (&h->lock);
  return ret;
}

const char *
nbd_connection_state (struct nbd_handle *h)
{
  const char *ret;

  nbd_internal_reset_error ("nbd_connection_state");

  pthread_mutex_lock (&h->lock);
  ret = nbd_unlocked_connection_state (h);
  pthread_mutex_unlock (&h->lock);
  return ret;
}

const char *
nbd_get_package_name (struct nbd_handle *h)
{
  const char *ret;

  ret = nbd_unlocked_get_package_name (h);
  return ret;
}

const char *
nbd_get_version (struct nbd_handle *h)
{
  const char *ret;

  ret = nbd_unlocked_get_version (h);
  return ret;
}

int
nbd_supports_tls (struct nbd_handle *h)
{
  int ret;

  ret = nbd_unlocked_supports_tls (h);
  return ret;
}

int
nbd_supports_uri (struct nbd_handle *h)
{
  int ret;

  ret = nbd_unlocked_supports_uri (h);
  return ret;
}

